/*
 * Copyright 2016-2023 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file    LPC845_Project_PortTest1.c
 * @brief   Application entry point.
 */
#include <stdio.h>
#include "board.h"
#include "peripherals.h"
#include "pin_mux.h"
#include "clock_config.h"
#include "LPC845.h"
#include "fsl_debug_console.h"

/* TODO: insert other include files here. */
#include "fsl_gpio.h"	// ezt kézzel kell beírni (az előzők: automatikus)
/* TODO: insert other definitions and declarations here. */
#define bt GPIO_PinRead(GPIO, BOARD_K3_GPIO_PORT, BOARD_K3_GPIO_PIN)
#define LEDS_RED_BLUE_GREEN_MASK ~(uint32_t)0b111
#define wait_bt 0
#define pressed_bt 1
#define DEBUANCE_TIME 30U //prellmenetesítési idő
/*
 * @brief   Application entry point.
 */
volatile uint32_t g_systickCounter = DEBUANCE_TIME;

void SysTick_Handler(void) //Callback Systick Interrupt
{
    if (g_systickCounter != 0U)
    {
        g_systickCounter--;
    }
}


int main(void) {
	unsigned char state = wait_bt;
	volatile uint32_t value;
	int count = 0;
    /* Init board hardware. */
    BOARD_InitBootPins();
    BOARD_InitBootClocks();
    BOARD_InitBootPeripherals();
#ifndef BOARD_INIT_DEBUG_CONSOLE_PERIPHERAL
    /* Init FSL debug console. */
    BOARD_InitDebugConsole();
#endif
    /* Set systick reload value to generate 1ms interrupt */
    if (SysTick_Config(SystemCoreClock / 1000U)) // Set systick reload value to 1ms IT
    {
        while (1) //  If function failed infinite loop
        {
        }
    }
    g_systickCounter = DEBUANCE_TIME;
    PRINTF("Hello World\n");

    /* Force the counter to be placed into memory. */
    /* Enter an infinite loop, just incrementing a counter. */
    GPIO_PortMaskedSet(GPIO, BOARD_LED_GREEN_GPIO_PORT, LEDS_RED_BLUE_GREEN_MASK);
    while(1) {
		if(g_systickCounter == 0U)
		{
			g_systickCounter = DEBUANCE_TIME;
			switch(state)
			{
				case wait_bt:
				{
					if(!bt)						// pressed bt
					{
						value = ~(1 << count);
						GPIO_PortMaskedWrite(GPIO, BOARD_LED_GREEN_GPIO_PORT, value);
						state = pressed_bt;
						count = (count + 1)%3;
					}
					break;
				}
				case pressed_bt:
				{
					if(bt)					// released bt
					{
						state = wait_bt;
					}
					break;
				}
			}
		}
    }
    return 0;
}
