/*
 * Copyright 2016-2023 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file    LPC845_Project_PWM_LED.c
 * @brief   Application entry point.
 */
#include <stdio.h>
#include "board.h"
#include "peripherals.h"
#include "pin_mux.h"
#include "clock_config.h"
#include "LPC845.h"
#include "fsl_debug_console.h"
/* TODO: insert other include files here. */
#include "fsl_ctimer.h"
/* TODO: insert other definitions and declarations here. */

void CTIMER_UpdatePwmDutycycleFloat(CTIMER_Type *base,
                               const ctimer_match_t pwmPeriodChannel,
                               ctimer_match_t matchChannel,
                               float dutyCyclePercent)
{
    uint32_t pulsePeriod = 0, period;

    /* Specified channel pwmPeriodChannel  defines the PWM period */
    period = base->MR[pwmPeriodChannel];

    /* For 0% dutycyle, make pulse period greater than period so the event will never occur */
    if (dutyCyclePercent == 0)
    {
        pulsePeriod = period + 1U;
    }
    else
    {
        pulsePeriod = (uint32_t)(((float)period * ((float)100U - dutyCyclePercent)) / (float)100U + 0.5);
    }

    /* Update dutycycle */
    base->MR[matchChannel] = pulsePeriod;
}
volatile static uint8_t pwm_precent = 10;
volatile static float pwm_percent_float = 10;
void ctimer0_pwm_callback(uint32_t flags)
{
	float delta = 0.4;
	CTIMER_UpdatePwmDutycycleFloat(CTIMER0,
								   3U,					//pwmPeriodChannel ez csak a 3-as lehet!
	                               0U,					//matchChannel (0,1,2 lehet) max. 3db PWM kimenetlehet
								   (100-pwm_percent_float));	//dutyCyclePercent
	//pwm_precent = (pwm_precent + 1)%100;
	if(pwm_percent_float+ delta < 100)
	{
		pwm_percent_float = pwm_percent_float + delta;
	}
	else
	{
		pwm_percent_float = 0;
	}


}
/*
 * @brief   Application entry point.
 */
int main(void) {

    /* Init board hardware. */
    BOARD_InitBootPins();
    BOARD_InitBootClocks();
    BOARD_InitBootPeripherals();
#ifndef BOARD_INIT_DEBUG_CONSOLE_PERIPHERAL
    /* Init FSL debug console. */
    BOARD_InitDebugConsole();
#endif

    PRINTF("Hello World\n");

    /* Force the counter to be placed into memory. */
    volatile static int i = 0 ;
    /* Enter an infinite loop, just incrementing a counter. */
    while(1) {
        i++ ;
        /* 'Dummy' NOP to allow source level single stepping of
            tight while() loop */
        __asm volatile ("nop");
    }
    return 0 ;
}
