/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/

#include <stdlib.h>
//#include "project.h"
//#include "general.h"
#include "fsl_i2c.h"
#include "HMC5883L_Compass.h"

static double hmc5883l_scale = 0;

#define txBuffSize 2
#define rxBuffSize 6
uint8_t txBuff[txBuffSize];
uint8_t rxBuff[rxBuffSize];

/*
void I2C_RegWrite(uint8 SLA, uint8 reg_addr, uint8 data)
{
    I2C_MasterSendStart(SLA,0);
    I2C_MasterWriteByte(reg_addr);
    I2C_MasterWriteByte(data);
    I2C_MasterSendStop();    
}
*/
// Write I2C address, reg address, and reg data
uint8_t I2C_RegWrite(uint8_t SLA, uint8_t reg_addr, uint8_t data)
{
	uint8_t err = 0;
	int32_t status;
	txBuff[0] = reg_addr;
	txBuff[1] = data;
	status = I2C_MasterStart(I2C0, SLA, kI2C_Write);
	if(status == kStatus_Success)
	{
		status =I2C_MasterWriteBlocking(I2C0, txBuff, 2, kI2C_TransferDefaultFlag);
		if(status != kStatus_Success){ err = 1; }
	}
	else{ err = 1; }
	//STOP genertated by I2C_MasterWriteBlocking()
	return(err);
}

/*
void I2C_SetRegAddrToRD(uint8 SLA, uint8 reg_addr)
{
    I2C_MasterSendStart(SLA,0);
    I2C_MasterWriteByte(reg_addr);
    I2C_MasterSendStop();    
}
*/
// Write I2C address, and set reg address
uint8_t I2C_SetRegAddrToRD(uint8_t SLA, uint8_t reg_addr)
{
	uint8_t err = 0;
	int32_t status;
	txBuff[0] = reg_addr;
	status = I2C_MasterStart(I2C0, SLA, kI2C_Write);
	if(status == kStatus_Success)
	{
		status = I2C_MasterWriteBlocking(I2C0, txBuff, 1, kI2C_TransferDefaultFlag);
		if(status != kStatus_Success){ err = 1;}
	}else{ err = 1; }
	return(err);
}

/*
void I2C_RegReadBuf(uint8 SLA, uint8 reag_addr, uint8 * buf, uint8 count)
{
    I2C_SetRegAddrToRD(SLA, reag_addr);
    I2C_MasterClearStatus();
    I2C_MasterReadBuf(SLA, buf, count, I2C_MODE_COMPLETE_XFER);    
    do{
    }while(!(I2C_MasterStatus()&I2C_MSTAT_RD_CMPLT));    
}
*/

//Write I2C address, set reg address, read reg data
uint8_t I2C_RegReadBuf(uint8_t SLA, uint8_t reag_addr, uint8_t * buf, uint8_t count)
{
	uint32_t flags = 0;
	uint32_t status;
	uint8_t err = 0;
	err = I2C_SetRegAddrToRD(SLA, reag_addr);
	if(!err)
	{
		status = I2C_MasterStart(I2C0, SLA, kI2C_Read);
	}
	if(status == kStatus_Success)
	{
		status = I2C_MasterReadBlocking(I2C0, rxBuff, count, flags);
		if(status != kStatus_Success){ err = 1; }
	}
	else{ err = 1; }
	return(err);
}

// init the hmc5883l

uint8_t hmc5883l_init(void) {
	uint8_t err = 0;
	#if HMC5883L_I2CINIT == 1
	//init i2c
	I2C_Init();
	CyDelayUs(10);
	#endif

	//set scale
	hmc5883l_scale = 0;
	uint8_t scale = 0x00;
	#if HMC5883L_SCALE == HMC5883L_SCALE088
		scale = 0x00;
		hmc5883l_scale = 0.73;
	#elif HMC5883L_SCALE == HMC5883L_SCALE13
		scale = 0x01;
		hmc5883l_scale = 0.92;
	#elif HMC5883L_SCALE == HMC5883L_SCALE19
		regValue = 0x02;
		hmc5883l_scale = 1.22;
	#elif HMC5883L_SCALE == HMC5883L_SCALE25
		scale = 0x03;
		hmc5883l_scale = 1.52;
	#elif HMC5883L_SCALE == HMC5883L_SCALE40
		regValue = 0x04;
		hmc5883l_scale = 2.27;
	#elif HMC5883L_SCALE == HMC5883L_SCALE47
		scale = 0x05;
		hmc5883l_scale = 2.56;
	#elif HMC5883L_SCALE == HMC5883L_SCALE56
		regValue = 0x06;
		hmc5883l_scale = 3.03;
	#elif HMC5883L_SCALE == HMC5883L_SCALE81
		scale = 0x07;
		hmc5883l_scale = 4.35;
	#endif

	//setting is in the top 3 bits of the register.
	scale = scale << 5;
    
    err = I2C_RegWrite(HMC5883L_ADDR, HMC5883L_CONFREGB, scale);
    err = err + I2C_RegWrite(HMC5883L_ADDR, HMC5883L_MODEREG, HMC5883L_MEASURECONTINOUS);
    return(err);
}


//get raw data
uint8_t hmc5883l_getrawdata(int16_t *mxraw, int16_t *myraw, int16_t *mzraw) {
    #define dat_num 6
	uint8_t err;
	uint8_t buff[dat_num];
 
    err = I2C_RegReadBuf(HMC5883L_ADDR, HMC5883L_DATAREGBEGIN, &buff[0], dat_num);
    
	*mxraw = (int16_t)((buff[0] << 8) | buff[1]);
	*mzraw = (int16_t)((buff[2] << 8) | buff[3]);
	*myraw = (int16_t)((buff[4] << 8) | buff[5]);
	return(err);
}


// get scaled data

uint8_t hmc5883l_getdata(double *mx, double *my, double *mz)
 {
	uint8_t err;
	int16_t mxraw = 0;
	int16_t myraw = 0;
	int16_t mzraw = 0;
	err = hmc5883l_getrawdata(&mxraw, &myraw, &mzraw);

	#if HMC5883L_CALIBRATED == 1
	float mxt = mxraw - HMC5883L_OFFSETX;
	float myt = myraw - HMC5883L_OFFSETY;
	float mzt = mzraw - HMC5883L_OFFSETZ;
	*mx = HMC5883L_GAINX1 * mxt + HMC5883L_GAINX2 * myt + HMC5883L_GAINX3 * mzt;
	*my = HMC5883L_GAINY1 * mxt + HMC5883L_GAINY2 * myt + HMC5883L_GAINY3 * mzt;
	*mz = HMC5883L_GAINZ1 * mxt + HMC5883L_GAINZ2 * myt + HMC5883L_GAINZ3 * mzt;

	#else
	*mx = mxraw * hmc5883l_scale;
	*my = myraw * hmc5883l_scale;
	*mz = mzraw * hmc5883l_scale;
	#endif

return(err);
}

/* [] END OF FILE */
