/*
 * Copyright 2016-2023 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file    LPC845_Project_I2C_Compass.c
 * @brief   Application entry point.
 */
#include <stdio.h>
#include "board.h"
#include "peripherals.h"
#include "pin_mux.h"
#include "clock_config.h"
#include "LPC845.h"
#include "fsl_debug_console.h"

/* TODO: insert other include files here. */
#include "Sys.h"
#include "SysTimer.h"
#include "HMC5883L_Compass.h"
#include "math.h"

/* TODO: insert other definitions and declarations here. */

/*
 * @brief   Application entry point.
 */

/*
 // I2C START és I2C address és irány:
  //param base I2C peripheral base pointer
  //param address 7-bit slave device address.
  //param direction Master transfer directions(transmit/receive): kI2C_Write, kI2C_Read
  //retval kStatus_Success Successfully send the start signal.
  //retval kStatus_I2C_Busy Current bus is busy.
  status_t I2C_MasterStart(I2C_Type *base, uint8_t address, i2c_direction_t direction)
*/
/*
 // adat tömb írása:
 * brief Performs a polling send transfer on the I2C bus.
 * Sends up to a txSize number of bytes to the previously addressed slave device. The slave may
 * reply with a NAK to any byte in order to terminate the transfer early. If this happens, this
 * function returns #kStatus_I2C_Nak.
 *
 * param base  The I2C peripheral base address.
 * param txBuff The pointer to the data to be transferred.
 * param txSize The length in bytes of the data to be transferred.
 * param flags Transfer control flag to control special behavior like suppressing start or stop, for normal transfers
 * use kI2C_TransferDefaultFlag
 * retval kStatus_Success Data was sent successfully.
 * retval #kStatus_I2C_Busy Another master is currently utilizing the bus.
 * retval #kStatus_I2C_Nak The slave device sent a NAK in response to a byte.
 * retval #kStatus_I2C_ArbitrationLost Arbitration lost error.
 status_t I2C_MasterWriteBlocking(I2C_Type *base, const void *txBuff, size_t txSize, uint32_t flags)
*/

/*
 // adat tömb olvasása:
 * brief Performs a polling receive transfer on the I2C bus.
 *
 * param base  The I2C peripheral base address.
 * param rxBuff The pointer to the data to be transferred.
 * param rxSize The length in bytes of the data to be transferred.
 * param flags Transfer control flag to control special behavior like suppressing start or stop, for normal transfers
 * use kI2C_TransferDefaultFlag
 * retval kStatus_Success Data was received successfully.
 * retval #kStatus_I2C_Busy Another master is currently utilizing the bus.
 * retval #kStatus_I2C_Nak The slave device sent a NAK in response to a byte.
 * retval #kStatus_I2C_ArbitrationLost Arbitration lost error.
 status_t I2C_MasterReadBlocking(I2C_Type *base, void *rxBuff, size_t rxSize, uint32_t flags)
*/
/*
// I2C STOP
  //brief Sends a STOP signal on the I2C bus.
  //retval kStatus_Success Successfully send the stop signal.
  //retval kStatus_I2C_Timeout Send stop signal failed, timeout.
  status_t I2C_MasterStop(I2C_Type *base)
 */

int main(void) {
	char str[32];
	uint8_t err = 0;
	int16_t mxraw = 0;
	int16_t myraw = 0;
	int16_t mzraw = 0;

    /* Init board hardware. */
    BOARD_InitBootPins();
    BOARD_InitBootClocks();
    BOARD_InitBootPeripherals();
#ifndef BOARD_INIT_DEBUG_CONSOLE_PERIPHERAL
    /* Init FSL debug console. */
    BOARD_InitDebugConsole();
#endif
    SysTickInit(1); // 1ms
    err = hmc5883l_init();
    err = err + hmc5883l_getrawdata(&mxraw, &myraw, &mzraw);

    if(err)
    {
    	PRINTF("Compass init error! %d\r\n", err);
    	while(err);
    }
    /* Force the counter to be placed into memory. */

    /* Enter an infinite loop, just incrementing a counter. */
    StartTimeout1(1000);
    while(1) {
    	if(TimeoutDone1())
    	{
    		hmc5883l_getrawdata(&mxraw, &myraw, &mzraw);
    		sprintf(str, "x: %d,       ",mxraw);
    		PRINTF(str);
    		sprintf(str, "y: %d,       ",myraw);
    		PRINTF(str);
    		sprintf(str, "z: %d\r\n",mzraw);
    		PRINTF(str);
    	    StartTimeout1(1000);
    	}
    }
    return 0 ;
}
