// EMC1001 teperature sensor handling routines

#include "fsl_i2c.h"
#include"I2C.h"
#include "BooleConst.h"
//#include "Timer_std_hlr.h"

//I2C address select pull up resistor: 12k
#define EMC1001addr 0x49
#define EMC1001wra (EMC1001addr << 1)
#define EMC1001rda ((EMC1001addr << 1) | 0x01)

//EMC1001 internal reg. addresses:
#define tempH_reg		0x00

//7		6		5		0
//Busy	THL		TLL		Therm
#define stat_reg		0x01

#define tempL_reg		0x02

//7				6				5
//AlDis Standby/cont.run	Therm2/Alert
#define conf_reg		0x03

//conv rate: 9 32Hz, 8 16Hz, 7 8Hz, 6 4Hz, 5 2Hz, 4 1Hz 3 0.5Hz, 2 0.25Hz, 1 0.125Hz, 0 0.0625Hz
#define crate_reg		0x04

#define HlimH_reg		0x05
#define HlimL_reg		0x06
#define LlimH_reg		0x07
#define LlimL_reg		0x08
#define oneshot_reg		0x0f
#define ThermLim_reg	0x20
#define ThermHis_reg	0x21
#define SMtou_reg		0x22
#define ProdID_reg		0xfd
#define ManID_reg		0xfe
#define RevNum_reg		0xff

#define Busy            0x80

/*
void EMC1001_Ini(unsigned char rate)
{
	unsigned char rdy;
	do{
		rdy = StartCond();
		rdy &= TWI_DATA_WRITE(EMC1001wra);	//I2C WR address
		rdy &= TWI_DATA_WRITE(crate_reg);	//EMC1001 crate reg. address
		rdy &= TWI_DATA_WRITE(rate);		//EMC1001(crate_reg) = data
		StopCond();
	}while(!rdy);
}
*/
//Set rate
uint8_t EMC1001_Ini(unsigned char rate)
{
	uint8_t err = I2C_RegWrite(EMC1001addr, crate_reg, rate);
	return(err);
}


unsigned char EMC1001_RDY(void)
{
	uint8_t err, temp_stat;
	
	err = I2C_RegReadBuf(EMC1001addr, stat_reg, &temp_stat, 1);
	
	if(!err && !(temp_stat & Busy))
	{
		return(TRUE);
	}
	else
	{
		return(FALSE);
	}
}

unsigned char temph;
unsigned char templ;
unsigned int tempi;
int temp;
unsigned char sig;
double tempd;

// get temperature from sensor
/*
unsigned char Temperature(double *t)
{
	unsigned char rdy;
	// set tempH address
	if(EMC1001_RDY())
	{
		do{
			rdy = StartCond();
			rdy &= TWI_DATA_WRITE(EMC1001wra);	//I2C WR address
			rdy &= TWI_DATA_WRITE(tempH_reg);	//EMC1001 tempH_reg address
			StopCond();	
		}while(!rdy);

		// read tempH	
		do{	
			rdy = StartCond();
			rdy &= TWI_DATA_WRITE(EMC1001rda);	//I2C RD address
		}while(!rdy);	
		rdy = TWI_DATA_READ(&temph,0);			//temph = EMC1001(tempH_reg), master NACK (last byte)
		StopCond();

		//DelayTim0(1);
		// set tempL address
		do{
			rdy = StartCond();
			rdy &= TWI_DATA_WRITE(EMC1001wra);	//I2C WR address
			rdy &= TWI_DATA_WRITE(tempL_reg);	//EMC1001 tempL_reg address
			StopCond();		
		}while(!rdy);
	
		// read tempL
		do{
			rdy = StartCond();	
			rdy &= TWI_DATA_WRITE(EMC1001rda);	//I2C RD address
		}while(!rdy);
		rdy = TWI_DATA_READ(&templ,0);			//tempL = EMC1001(tempL_reg), master NACK (last byte)
		StopCond();

		sig = temph & 0x80;			//sign of temperature
		tempi = (((unsigned int)temph << 8) | (unsigned int)templ) >> 6;	// tempi = 4*temperatute (the last 2 bits are the fractional part of the fix point number)
		if(sig)
		{
			tempi |= 0xfc00;	// if temperature < 0 must be 2-s complement sign extension 
		}

		*t = (double)tempi/4;	// calculate the temperature and convert to float
		return(TRUE);
	}
	else
	{
		return(FALSE);
	}
}
*/
uint8_t Temperature(double *t)
{
	unsigned char temph;
	unsigned char templ;
	unsigned int tempi;
	uint8_t sig, err;

	if(EMC1001_RDY())
	{
		err = I2C_RegReadBuf(EMC1001addr, tempH_reg, &temph, 1);
		err = err + I2C_RegReadBuf(EMC1001addr, tempL_reg, &templ, 1);
		if(!err)
		{
			sig = temph & 0x80;			//sign of temperature
			tempi = (((unsigned int)temph << 8) | (unsigned int)templ) >> 6;	// tempi = 4*temperatute (the last 2 bits are the fractional part of the fix point number)
			if(sig)
			{
				tempi |= 0xfc00;	// if temperature < 0 must be 2-s complement sign extension
			}

			*t = (double)tempi/4;	// calculate the temperature and convert to float
			return(TRUE);
		}
		else{return(FALSE);}
	}
	else
	{
		return(FALSE);
	}
}



