/*
 * Copyright 2016-2023 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file    LPC845_Project_ADC.c
 * @brief   Application entry point.
 */
#include <stdio.h>
#include "board.h"
#include "peripherals.h"
#include "pin_mux.h"
#include "clock_config.h"
#include "LPC845.h"
#include "fsl_debug_console.h"
/* TODO: insert other include files here. */
#include "fsl_power.h"
#include "fsl_adc.h"
/* TODO: insert other definitions and declarations here. */
static adc_result_info_t gAdcResultInfoStruct;
adc_result_info_t *volatile gAdcResultInfoPtr = &gAdcResultInfoStruct;
volatile bool gAdcConvSeqAIntFlag;
const uint32_t g_Adc_12bitFullRange = 4096U;
/*
 * @brief   Application entry point.
 */

/*!
 * brief Get the channel's ADC conversion completed under each conversion sequence.
 *
 * param base ADC peripheral base address.
 * param channel The indicated channel number.
 * param info Pointer to information structure, see to #adc_result_info_t;
 * retval true  The conversion result is ready.
 * retval false The conversion result is not ready yet.
 */
/*
bool ADC_GetChannelConversionResult(ADC_Type *base, uint32_t channel, adc_result_info_t *info)
typedef struct _adc_result_info
{
    uint32_t result;                                         //!< Keep the conversion data value.
    adc_threshold_compare_status_t thresholdCompareStatus;   //!< Keep the threshold compare status.
    adc_threshold_crossing_status_t thresholdCorssingStatus; //!< Keep the threshold crossing status.
    uint32_t channelNumber;                                  //!< Keep the channel number for this conversion.
    bool overrunFlag; //!< Keep the status whether the conversion is overrun or not.
    //The data available flag would be returned by the reading result API.
} adc_result_info_t;
*/

/* ADC0_SEQA_IRQn interrupt handler */
void ADC0_ADC_SEQ_A_IRQHANDLER(void) {
  /* Get status flags */
volatile uint32_t stat = 0;
bool ADC_result_rdy = false;
	stat = ADC_GetStatusFlags(ADC0_PERIPHERAL);
  if (kADC_ConvSeqAInterruptFlag == (kADC_ConvSeqAInterruptFlag & stat))
  {
  	/* Place your interrupt code here */
	  ADC_result_rdy = ADC_GetChannelConversionResult(ADC0, 0U, gAdcResultInfoPtr);
  	/* Clear status flags */
  	ADC_ClearStatusFlags(ADC0_PERIPHERAL, kADC_ConvSeqAInterruptFlag);
  	gAdcConvSeqAIntFlag = ADC_result_rdy;
  }

  /* Add for ARM errata 838869, affects Cortex-M4, Cortex-M4F
     Store immediate overlapping exception return operation might vector to incorrect interrupt. */
  #if defined __CORTEX_M && (__CORTEX_M == 4U)
    __DSB();
  #endif
}


int main(void) {
uint8_t OK = 0;
uint32_t result = 0;
float U;
char str[16];

    /* Init board hardware. */
    BOARD_InitBootPins();
    BOARD_InitBootClocks();

// CLOCK_EnableClock(kclock_Adc);
//? kPDAWAKECFG_Wakeup_VREFF2
    POWER_DisablePD(kPDRUNCFG_PD_ADC0);    // ADC táp bekapcs.
    OK = ADC_DoSelfCalibration(ADC0_PERIPHERAL, ADC0_CLK_FREQ);
    BOARD_InitBootPeripherals();
    /*
    //Clear the result register.
    ADC_DisableInterrupts(ADC0_PERIPHERAL, kADC_ConvSeqAInterruptEnable);
    ADC_DoSoftwareTriggerConvSeqA(ADC0_PERIPHERAL);
    while (!ADC_GetChannelConversionResult(ADC0_PERIPHERAL, 0U, gAdcResultInfoPtr))
    {
    }
    ADC_GetConvSeqAGlobalConversionResult(ADC0_PERIPHERAL, &gAdcResultInfoStruct);
    ADC_EnableInterrupts(ADC0_PERIPHERAL, kADC_ConvSeqAInterruptEnable);
    */

#ifndef BOARD_INIT_DEBUG_CONSOLE_PERIPHERAL
    /* Init FSL debug console. */
    BOARD_InitDebugConsole();
#endif

    if(OK)
    {
    	PRINTF("ADC calibrated.\r\n");
    }
    else
    {
    	PRINTF("ADC calibration faild!\r\n");
    }

    /* Enter an infinite loop, just incrementing a counter. */
    while(1) {
         //GETCHAR();
         gAdcConvSeqAIntFlag = false;
         //ADC_DoSoftwareTriggerConvSeqA(ADC0_PERIPHERAL);
         while (!gAdcConvSeqAIntFlag){}
         result = gAdcResultInfoStruct.result;
         U = (float)result*3.3/4095;
         PRINTF("code: %d    ", result);
         sprintf(str, "U: %.2f V\r\n", U);
         PRINTF(str);
    }
    return 0 ;
}
